require 'puppet/resource_api'

Puppet::ResourceApi.register_type(
  name: 'dsc_webapplicationhandler',
  dscmeta_resource_friendly_name: 'WebApplicationHandler',
  dscmeta_resource_name: 'MSFT_WebApplicationHandler',
  dscmeta_resource_implementation: 'MOF',
  dscmeta_module_name: 'xWebAdministration',
  dscmeta_module_version: '3.2.0',
  docs: 'The DSC WebApplicationHandler resource type.
         Automatically generated from version 3.2.0',
  features: ['simple_get_filter', 'canonicalize', 'custom_insync'],
  attributes: {
    name: {
      type:      'String',
      desc:      'Description of the purpose for this resource declaration.',
      behaviour: :namevar,
    },
    validation_mode: {
      type:      'Enum[property, resource]',
      desc:      'Whether to check if the resource is in the desired state by property (default) or using Invoke-DscResource in Test mode (resource).',
      behaviour: :parameter,
      default:   'property',
    },
    dsc_resourcetype: {
      type: 'Optional[String]',
      desc: 'Specifies the resource type this handler runs.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_responsebufferlimit: {
      type: 'Optional[Integer[0, 4294967295]]',
      desc: 'Specifies the maximum size, in bytes, of the response buffer for a request handler runs.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'UInt32',
      mof_is_embedded: false,
    },
    dsc_type: {
      type: 'Optional[String]',
      desc: 'Specifies the managed type of the new module. This parameter applies to managed modules only.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_modules: {
      type: 'Optional[String]',
      desc: 'Specifies the modules used for the handler.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_name: {
      type: 'String',
      desc: 'Specifies the name of the new request handler.',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_ensure: {
      type: "Optional[Enum['Present', 'Absent']]",
      desc: "Indicates if the application handler exists. Set this property to `Absent` to ensure that the application handler does not exist. Default value is 'Present'.",

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_psdscrunascredential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: ' ',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_precondition: {
      type: 'Optional[String]',
      desc: 'Specifies preconditions for the new handler.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_physicalhandlerpath: {
      type: 'Optional[String]',
      desc: 'Specifies the physical path to the handler. This parameter applies to native modules only.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_scriptprocessor: {
      type: 'Optional[String]',
      desc: 'Specifies the script processor that runs for the module.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_allowpathinfo: {
      type: 'Optional[Boolean]',
      desc: 'Specifies whether the handler processes full path information in a URI, such as contoso/marketing/imageGallery.aspx. If the value is true, the handler processes the full path, contoso/marketing/imageGallery. If the value is false, the handler processes only the last section of the path, /imageGallery.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_location: {
      type: 'Optional[String]',
      desc: 'Specifies The location of the configuration setting. Location tags are frequently used for configuration settings that must be set more precisely than per application or per virtual directory.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_path: {
      type: 'Array[String]',
      desc: 'Specifies an IIS configuration path.',

      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String[]',
      mof_is_embedded: false,
    },
    dsc_requireaccess: {
      type: "Optional[Enum['None', 'Read', 'Write', 'Script', 'Execute']]",
      desc: 'Specifies the user rights that are required for the new handler.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_verb: {
      type: 'Optional[String]',
      desc: 'Specifies the HTTP verbs that are handled by the new handler.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
  },
)
