require 'puppet/resource_api'

Puppet::ResourceApi.register_type(
  name: 'dsc_xiislogging',
  dscmeta_resource_friendly_name: 'xIisLogging',
  dscmeta_resource_name: 'MSFT_xIisLogging',
  dscmeta_resource_implementation: 'MOF',
  dscmeta_module_name: 'xWebAdministration',
  dscmeta_module_version: '3.2.0',
  docs: 'The DSC xIisLogging resource type.
         Automatically generated from version 3.2.0',
  features: ['simple_get_filter', 'canonicalize', 'custom_insync'],
  attributes: {
    name: {
      type:      'String',
      desc:      'Description of the purpose for this resource declaration.',
      behaviour: :namevar,
    },
    validation_mode: {
      type:      'Enum[property, resource]',
      desc:      'Whether to check if the resource is in the desired state by property (default) or using Invoke-DscResource in Test mode (resource).',
      behaviour: :parameter,
      default:   'property',
    },
    dsc_psdscrunascredential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: ' ',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_logtargetw3c: {
      type: "Optional[Enum['File', 'ETW', 'File,ETW']]",
      desc: 'Specifies whether IIS will use Event Tracing or file logging',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_logtruncatesize: {
      type: 'Optional[String]',
      desc: 'How large the file should be before it is truncated',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_logflags: {
      type: "Optional[Array[Enum['Date', 'Time', 'ClientIP', 'UserName', 'SiteName', 'ComputerName', 'ServerIP', 'Method', 'UriStem', 'UriQuery', 'HttpStatus', 'Win32Status', 'BytesSent', 'BytesRecv', 'TimeTaken', 'ServerPort', 'UserAgent', 'Cookie', 'Referer', 'ProtocolVersion', 'Host', 'HttpSubStatus']]]",
      desc: 'The W3C logging fields',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String[]',
      mof_is_embedded: false,
    },
    dsc_logperiod: {
      type: "Optional[Enum['Hourly', 'Daily', 'Weekly', 'Monthly', 'MaxSize']]",
      desc: 'How often the log file should rollover',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_logformat: {
      type: "Optional[Enum['IIS', 'W3C', 'NCSA']]",
      desc: 'Format of the Logfiles. Only W3C supports LogFlags',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_logcustomfields: {
      type: "Optional[Array[Struct[{
              sourcetype => Enum['RequestHeader', 'ResponseHeader', 'ServerVariable'],
              sourcename => String,
              ensure => Optional[Enum['Present', 'Absent']],
              logfieldname => String
            }]]]",
      desc: 'Custom logging field information in the form of an array of embedded instances of MSFT_xLogCustomField CIM class',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'MSFT_xLogCustomField[]',
      mof_is_embedded: true,
    },
    dsc_loglocaltimerollover: {
      type: 'Optional[Boolean]',
      desc: 'Use the localtime for file naming and rollover',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_logpath: {
      type: 'String',
      desc: 'The directory to be used for logfiles',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
  },
)
