require 'puppet/resource_api'

Puppet::ResourceApi.register_type(
  name: 'dsc_xsslsettings',
  dscmeta_resource_friendly_name: 'xSslSettings',
  dscmeta_resource_name: 'MSFT_xSslSettings',
  dscmeta_resource_implementation: 'MOF',
  dscmeta_module_name: 'xWebAdministration',
  dscmeta_module_version: '3.2.0',
  docs: 'The DSC xSslSettings resource type.
         Automatically generated from version 3.2.0',
  features: ['simple_get_filter', 'canonicalize', 'custom_insync'],
  attributes: {
    name: {
      type:      'String',
      desc:      'Description of the purpose for this resource declaration.',
      behaviour: :namevar,
    },
    validation_mode: {
      type:      'Enum[property, resource]',
      desc:      'Whether to check if the resource is in the desired state by property (default) or using Invoke-DscResource in Test mode (resource).',
      behaviour: :parameter,
      default:   'property',
    },
    dsc_bindings: {
      type: "Array[Enum['', 'Ssl', 'SslNegotiateCert', 'SslRequireCert', 'Ssl128']]",
      desc: 'The Bindings in which to modify for the website',

      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String[]',
      mof_is_embedded: false,
    },
    dsc_name: {
      type: 'String',
      desc: 'Name of website in which to modify the SSL Settings',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_psdscrunascredential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: ' ',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_ensure: {
      type: "Optional[Enum['Present', 'Absent']]",
      desc: 'Whether the bindings should be present or absent',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
  },
)
