require 'puppet/resource_api'

Puppet::ResourceApi.register_type(
  name: 'dsc_xwebconfigpropertycollection',
  dscmeta_resource_friendly_name: 'xWebConfigPropertyCollection',
  dscmeta_resource_name: 'MSFT_xWebConfigPropertyCollection',
  dscmeta_resource_implementation: 'MOF',
  dscmeta_module_name: 'xWebAdministration',
  dscmeta_module_version: '3.2.0',
  docs: 'The DSC xWebConfigPropertyCollection resource type.
         Automatically generated from version 3.2.0',
  features: ['simple_get_filter', 'canonicalize', 'custom_insync'],
  attributes: {
    name: {
      type:      'String',
      desc:      'Description of the purpose for this resource declaration.',
      behaviour: :namevar,
    },
    validation_mode: {
      type:      'Enum[property, resource]',
      desc:      'Whether to check if the resource is in the desired state by property (default) or using Invoke-DscResource in Test mode (resource).',
      behaviour: :parameter,
      default:   'property',
    },
    dsc_psdscrunascredential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: ' ',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_itemkeyname: {
      type: 'String',
      desc: 'Name of the key of the property collection item to update.',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_ensure: {
      type: "Optional[Enum['Present', 'Absent']]",
      desc: 'Indicates if the property and value of the property collection item should be present or absent. Defaults to Present.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_itempropertyname: {
      type: 'String',
      desc: 'Name of the property of the property collection item to update.',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_itempropertyvalue: {
      type: 'Optional[String]',
      desc: 'Value of the property of the property collection item to update.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_websitepath: {
      type: 'String',
      desc: 'Path to website location (IIS or WebAdministration format).',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_collectionname: {
      type: 'String',
      desc: 'Name of the property collection to update.',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_filter: {
      type: 'String',
      desc: 'Filter used to locate property collection to update.',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_itemkeyvalue: {
      type: 'String',
      desc: 'Value of the key of the property collection item to update.',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_itemname: {
      type: 'String',
      desc: 'Name of the property collection item to update.',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
  },
)
