require 'puppet/resource_api'

Puppet::ResourceApi.register_type(
  name: 'dsc_xwebsite',
  dscmeta_resource_friendly_name: 'xWebSite',
  dscmeta_resource_name: 'MSFT_xWebSite',
  dscmeta_resource_implementation: '',
  dscmeta_module_name: 'xWebAdministration',
  dscmeta_module_version: '3.2.0',
  docs: 'The DSC xWebSite resource type.
         Automatically generated from version 3.2.0',
  features: ['simple_get_filter', 'canonicalize', 'custom_insync'],
  attributes: {
    name: {
      type:      'String',
      desc:      'Description of the purpose for this resource declaration.',
      behaviour: :namevar,
    },
    validation_mode: {
      type:      'Enum[property, resource]',
      desc:      'Whether to check if the resource is in the desired state by property (default) or using Invoke-DscResource in Test mode (resource).',
      behaviour: :parameter,
      default:   'property',
    },
    dsc_serviceautostartprovider: {
      type: 'Optional[String]',
      desc: 'Adds a AutostartProvider',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_psdscrunascredential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: ' ',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_name: {
      type: 'String',
      desc: 'The desired name of the website.',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_preloadenabled: {
      type: 'Optional[Boolean]',
      desc: 'When set to $true this will allow WebSite to automatically start without a request',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_logformat: {
      type: "Optional[Enum['IIS', 'W3C', 'NCSA']]",
      desc: 'Format of the Logfiles. Only W3C supports LogFlags',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_logperiod: {
      type: "Optional[Enum['Hourly', 'Daily', 'Weekly', 'Monthly', 'MaxSize']]",
      desc: 'How often the log file should rollover',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_siteid: {
      type: 'Optional[Integer[0, 4294967295]]',
      desc: 'Optional. The desired IIS site Id for the website.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'UInt32',
      mof_is_embedded: false,
    },
    dsc_ensure: {
      type: "Optional[Enum['Present', 'Absent']]",
      desc: 'Ensures that the website is Present or Absent. Defaults to Present.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_logtargetw3c: {
      type: "Optional[Enum['File', 'ETW', 'File,ETW']]",
      desc: 'Specifies whether IIS will use Event Tracing or file logging',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_logtruncatesize: {
      type: 'Optional[String]',
      desc: 'How large the file should be before it is truncated',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_serviceautostartenabled: {
      type: 'Optional[Boolean]',
      desc: 'When set to `$true` this will enable application Autostart (application initalization without an initial request) on a Website',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_bindinginfo: {
      type: "Optional[Array[Struct[{
              sslflags => Optional[Enum['0', '1', '2', '3']],
              certificatestorename => Optional[Enum['My', 'WebHosting']],
              certificatethumbprint => Optional[String],
              hostname => Optional[String],
              certificatesubject => Optional[String],
              bindinginformation => Optional[String],
              port => Optional[Integer[0, 65535]],
              ipaddress => Optional[String],
              protocol => Enum['http', 'https', 'msmq.formatname', 'net.msmq', 'net.pipe', 'net.tcp']
            }]]]",
      desc: "Website's binding information in the form of an array of embedded instances of the MSFT_xWebBindingInformation CIM class.",

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'MSFT_xWebBindingInformation[]',
      mof_is_embedded: true,
    },
    dsc_applicationpool: {
      type: 'Optional[String]',
      desc: 'The name of the websiteâ??s application pool.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_physicalpath: {
      type: 'Optional[String]',
      desc: 'The path to the files that compose the website.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_defaultpage: {
      type: 'Optional[Array[String]]',
      desc: 'One or more names of files that will be set as Default Documents for this website.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String[]',
      mof_is_embedded: false,
    },
    dsc_logflags: {
      type: "Optional[Array[Enum['Date', 'Time', 'ClientIP', 'UserName', 'SiteName', 'ComputerName', 'ServerIP', 'Method', 'UriStem', 'UriQuery', 'HttpStatus', 'Win32Status', 'BytesSent', 'BytesRecv', 'TimeTaken', 'ServerPort', 'UserAgent', 'Cookie', 'Referer', 'ProtocolVersion', 'Host', 'HttpSubStatus']]]",
      desc: 'The W3C logging fields',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String[]',
      mof_is_embedded: false,
    },
    dsc_applicationtype: {
      type: 'Optional[String]',
      desc: 'Adds a AutostartProvider ApplicationType',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_state: {
      type: "Optional[Enum['Started', 'Stopped']]",
      desc: 'The state of the website.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_loglocaltimerollover: {
      type: 'Optional[Boolean]',
      desc: 'Use the localtime for file naming and rollover',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_serverautostart: {
      type: 'Optional[Boolean]',
      desc: 'When set to $true this will enable Autostart on a Website',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_enabledprotocols: {
      type: 'Optional[String]',
      desc: 'The protocols that are enabled for the website.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_logpath: {
      type: 'Optional[String]',
      desc: 'The directory to be used for logfiles',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_authenticationinfo: {
      type: 'Optional[Struct[{
              basic => Optional[Boolean],
              anonymous => Optional[Boolean],
              digest => Optional[Boolean],
              windows => Optional[Boolean]
            }]]',
      desc: 'Hashtable containing authentication information (Anonymous, Basic, Digest, Windows)',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'MSFT_xWebAuthenticationInformation',
      mof_is_embedded: true,
    },
    dsc_logcustomfields: {
      type: "Optional[Array[Struct[{
              sourcetype => Optional[Enum['RequestHeader', 'ResponseHeader', 'ServerVariable']],
              sourcename => Optional[String],
              logfieldname => Optional[String],
            }]]]",
      desc: 'Custom logging field information in the form of an array of embedded instances of MSFT_xLogCustomFieldInformation CIM class',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'MSFT_xLogCustomFieldInformation[]',
      mof_is_embedded: true,
    },
  },
)
