require 'puppet/resource_api'

Puppet::ResourceApi.register_type(
  name: 'dsc_xwebvirtualdirectory',
  dscmeta_resource_friendly_name: 'xWebVirtualDirectory',
  dscmeta_resource_name: 'MSFT_xWebVirtualDirectory',
  dscmeta_resource_implementation: 'MOF',
  dscmeta_module_name: 'xWebAdministration',
  dscmeta_module_version: '3.2.0',
  docs: 'The DSC xWebVirtualDirectory resource type.
         Automatically generated from version 3.2.0',
  features: ['simple_get_filter', 'canonicalize', 'custom_insync'],
  attributes: {
    name: {
      type:      'String',
      desc:      'Description of the purpose for this resource declaration.',
      behaviour: :namevar,
    },
    validation_mode: {
      type:      'Enum[property, resource]',
      desc:      'Whether to check if the resource is in the desired state by property (default) or using Invoke-DscResource in Test mode (resource).',
      behaviour: :parameter,
      default:   'property',
    },
    dsc_physicalpath: {
      type: 'String',
      desc: 'Physical path for the virtual directory',

      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_psdscrunascredential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: ' ',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_ensure: {
      type: "Optional[Enum['Present', 'Absent']]",
      desc: 'Whether virtual directory should be present or absent',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_webapplication: {
      type: 'String',
      desc: 'Web application name for the virtual directory',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_name: {
      type: 'String',
      desc: 'Name of virtual directory',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_website: {
      type: 'String',
      desc: 'Name of website with which Web Application is associated',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
  },
)
